'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.defaultConfigFolder = undefined;

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _command = require('@oclif/command');

var _config = require('./utils/config');

var _helpers = require('./utils/helpers');

var _print = require('./utils/print');

var _print2 = _interopRequireDefault(_print);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultConfigFolder = exports.defaultConfigFolder = '.lisk'; /*
                                                                    * LiskHQ/lisk-commander
                                                                    * Copyright © 2017–2018 Lisk Foundation
                                                                    *
                                                                    * See the LICENSE file at the top-level directory of this distribution
                                                                    * for licensing information.
                                                                    *
                                                                    * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
                                                                    * no part of this software, including this file, may be copied, modified,
                                                                    * propagated, or distributed except according to the terms contained in the
                                                                    * LICENSE file.
                                                                    *
                                                                    * Removal or modification of this copyright notice is prohibited.
                                                                    *
                                                                    */
class BaseCommand extends _command.Command {
	async init() {
		const { flags } = this.parse(this.constructor);
		this.flags = flags;

		process.on('error', _helpers.handleEPIPE);

		process.env.XDG_CONFIG_HOME = process.env.LISK_COMMANDER_CONFIG_DIR || `${_os2.default.homedir()}/${defaultConfigFolder}`;
		this.userConfig = (0, _config.getConfig)(process.env.XDG_CONFIG_HOME);
	}

	async finally(error) {
		if (error) {
			this.error(error.message ? error.message : error);
		}
	}

	print(result, readAgain = false) {
		if (readAgain) {
			this.userConfig = (0, _config.getConfig)(process.env.XDG_CONFIG_HOME);
		}
		(0, _print2.default)(Object.assign({
			json: this.userConfig.json,
			pretty: this.userConfig.pretty
		}, this.flags)).call(this, result);
	}
}

exports.default = BaseCommand;
const jsonDescription = 'Prints output in JSON format. You can change the default behaviour in your config.json file.';

const prettyDescription = 'Prints JSON in pretty format rather than condensed. Has no effect if the output is set to table. You can change the default behaviour in your config.json file.';

BaseCommand.flags = {
	json: _command.flags.boolean({
		char: 'j',
		description: jsonDescription,
		allowNo: true
	}),
	pretty: _command.flags.boolean({
		description: prettyDescription,
		allowNo: true
	})
};