'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _command = require('@oclif/command');

var _liskTransactions = require('@liskhq/lisk-transactions');

var _liskTransactions2 = _interopRequireDefault(_liskTransactions);

var _base = require('../../../base');

var _base2 = _interopRequireDefault(_base);

var _flags = require('../../../utils/flags');

var _flags2 = _interopRequireDefault(_flags);

var _input = require('../../../utils/input');

var _input2 = _interopRequireDefault(_input);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const processInputs = (amount, address) => ({ passphrase, secondPassphrase }) => _liskTransactions2.default.transfer({
	recipientId: address,
	amount,
	passphrase,
	secondPassphrase
}); /*
     * LiskHQ/lisk-commander
     * Copyright © 2017–2018 Lisk Foundation
     *
     * See the LICENSE file at the top-level directory of this distribution
     * for licensing information.
     *
     * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
     * no part of this software, including this file, may be copied, modified,
     * propagated, or distributed except according to the terms contained in the
     * LICENSE file.
     *
     * Removal or modification of this copyright notice is prohibited.
     *
     */
class TransferCommand extends _base2.default {
	async run() {
		const {
			args: { amount, address },
			flags: {
				passphrase: passphraseSource,
				'second-passphrase': secondPassphraseSource,
				'no-signature': noSignature
			}
		} = this.parse(TransferCommand);

		_liskTransactions2.default.utils.validateAddress(address);
		const normalizedAmount = _liskTransactions2.default.utils.convertLSKToBeddows(amount);
		const processFunction = processInputs(normalizedAmount, address);

		if (noSignature) {
			const result = processFunction({
				passphrase: null,
				secondPassphrase: null
			});
			return this.print(result);
		}

		const inputs = await (0, _input2.default)({
			passphrase: {
				source: passphraseSource,
				repeatPrompt: true
			},
			secondPassphrase: !secondPassphraseSource ? null : {
				source: secondPassphraseSource,
				repeatPrompt: true
			}
		});
		const result = processFunction(inputs);
		return this.print(result);
	}
}

exports.default = TransferCommand;
TransferCommand.args = [{
	name: 'amount',
	required: true,
	description: 'Amount of LSK to send.'
}, {
	name: 'address',
	required: true,
	description: 'Address of the recipient.'
}];

TransferCommand.flags = Object.assign({}, _base2.default.flags, {
	passphrase: _command.flags.string(_flags2.default.passphrase),
	'second-passphrase': _command.flags.string(_flags2.default.secondPassphrase),
	'no-signature': _command.flags.boolean(_flags2.default.noSignature)
});

TransferCommand.description = `
Creates a transaction which will transfer the specified amount to an address if broadcast to the network.
	`;

TransferCommand.examples = ['transaction:create:transfer 100 13356260975429434553L'];